@@ -28,7 +28,7 @@ legend_fill_color   = '#00000055'
 legend_text_color   = '#dddddd66'
 soft_colors = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']
 hard_colors = ['#000000', '#772211', '#000066', '#555555', '#0022cc', '#ffcc00']
-cmap_clash = pg.ColorMap([0.0, 0.2, 0.6, 1.0], [[0.5,0.5,1.0,0.2], [0.0,0.0,0.5,0.2], [1.0,0.2,0.4,0.2], [1.0,0.7,0.3,0.2]])
+cmap_clash = pg.ColorMap([0.0, 0.2, 0.6, 1.0], [[127, 127, 255, 51], [0, 0, 127, 51], [255, 51, 102, 51], [255, 178, 76, 51]])
 foreground = '#000000'
 background = '#ffffff'
 candle_bull_color = '#26a69a'
@@ -56,6 +56,7 @@ lod_labels = 700
 cache_candle_factor = 3 # factor extra candles rendered to buffer
 y_label_width = 65
 long_time = 2*365*24*60*60*1000
+display_timezone = None  # default to local
 winx,winy,winw,winh = 400,300,800,400
 
 app = None
@@ -90,7 +91,7 @@ class EpochAxisItem(pg.AxisItem):
         self.mode = 'num'
         ax = self.vb.parent()
         datasrc = _get_datasrc(ax, require=False)
-        if datasrc is None or not ax.x_indexed:
+        if datasrc is None or not self.vb.x_indexed:
             return super().tickValues(minVal, maxVal, size)
         # see if we have time
         self.mode = 'time'
@@ -215,17 +216,28 @@ class PandasDataSource:
         return len(self.df)+right_margin_candles
 
     def calc_significant_decimals(self):
-        absdiff = (self.z if len(self.scale_cols)>1 else self.y).diff().abs()
+        ser = self.z if len(self.scale_cols)>1 else self.y
+        absdiff = ser.diff().abs()
         absdiff[absdiff<1e-30] = 1e30
-        smallest_diff = absdiff.min()
-        s = '%.0e' % smallest_diff
-        exp = -int(s.partition('e')[2])
-        decimals = max(1, min(10, exp))
+        num = smallest_diff = absdiff.min()
+        for _ in range(2):
+            s = '%e' % num
+            base,_,exp = s.partition('e')
+            base = base.rstrip('0')
+            exp = -int(exp)
+            max_base_decimals = min(5, -exp+2) if exp < 0 else 3
+            base_decimals = max(0, min(max_base_decimals, len(base)-2))
+            decimals = exp + base_decimals
+            decimals = max(0, min(10, decimals))
+            if decimals <= 3:
+                break
+            # retry with full number, to see if we can get the number of decimals down
+            num = smallest_diff + abs(ser.min())
+        smallest_diff = max(10**(-decimals), smallest_diff)
         return decimals, smallest_diff
 
     def update_init_x(self, init_steps):
-        self.init_x0 = max(self.xlen-init_steps, 0) - side_margin
-        self.init_x1 = self.xlen - side_margin
+        self.init_x0, self.init_x1 = _xminmax(self, x_indexed=True, init_steps=init_steps)
 
     def closest_time(self, x):
         timecol = self.df.columns[0]
@@ -367,12 +379,14 @@ class FinWindow(pg.GraphicsLayoutWidget):
         self.centralWidget.installEventFilter(self)
         self.ci.setContentsMargins(0, 0, 0, 0)
         self.ci.setSpacing(-1)
+        self.closing = False
 
     @property
     def axs(self):
         return [ax for ax in self.ci.items if isinstance(ax, pg.PlotItem)]
 
     def close(self):
+        self.closing = True
         _savewindata(self)
         return super().close()
 
@@ -381,6 +395,10 @@ class FinWindow(pg.GraphicsLayoutWidget):
             _savewindata(self)
         return False
 
+    def leaveEvent(self, ev):
+        if not self.closing:
+            super().leaveEvent(ev)
+
 
 class FinCrossHair:
     def __init__(self, ax, color):
@@ -421,7 +439,7 @@ class FinCrossHair:
         rngmax = abs(self.ax.vb.y_min) + rng # any approximation is fine
         sd,se = (self.ax.significant_decimals,self.ax.significant_eps) if clamp_grid else (significant_decimals,significant_eps)
         timebased = False
-        if self.ax.x_indexed:
+        if self.ax.vb.x_indexed:
             xtext,timebased = _x2local_t(self.ax.vb.datasrc, x)
         else:
             xtext = _round_to_significant(rng, rngmax, x, sd, se)
@@ -429,7 +447,8 @@ class FinCrossHair:
         y = self.ax.vb.yscale.xform(y)
         ytext = _round_to_significant(rng, rngmax, y, sd, se)
         if not timebased:
-            xtext = 'x ' + xtext
+            if xtext:
+                xtext = 'x ' + xtext
             ytext = 'y ' + ytext
         far_right = self.ax.viewRect().x() + self.ax.viewRect().width()*0.9
         far_bottom = self.ax.viewRect().y() + self.ax.viewRect().height()*0.1
@@ -571,11 +590,13 @@ class FinViewBox(pg.ViewBox):
         self.y_max = 1000
         self.y_min = 0
         self.y_positive = True
+        self.x_indexed = True
         self.force_range_update = 0
         self.rois = []
         self.draw_line = None
         self.drawing = False
         self.set_datasrc(None)
+        self.standalones = []
         self.setMouseEnabled(x=True, y=False)
         self.init_steps = init_steps
         self.updating_linked = False
@@ -586,6 +607,13 @@ class FinViewBox(pg.ViewBox):
             return
         datasrc.update_init_x(self.init_steps)
 
+    @property
+    def datasrc_or_standalone(self):
+        ds = self.datasrc
+        if not ds and self.standalones:
+            ds = self.standalones[-1]
+        return ds
+
     def wheelEvent(self, ev, axis=None):
         if ev.modifiers() == QtCore.Qt.ControlModifier:
             scale_fact = 1
@@ -647,7 +675,7 @@ class FinViewBox(pg.ViewBox):
         p1 = _clamp_point(self.parent(), p1)
         def nonzerosize(a, b):
             c = b-a
-            return pg.Point(abs(c.x()) or 1, abs(c.y()) or 1)
+            return pg.Point(abs(c.x()) or 1, abs(c.y()) or 1e-3)
         if not self.drawing:
             # add new line
             p0 = self.mapToView(ev.lastPos())
@@ -707,8 +735,9 @@ class FinViewBox(pg.ViewBox):
                 if is_dirty or abs(vt0-tt0) >= period2 or abs(vt1-tt1) >= period2:
                     if is_dirty:
                         view.force_range_update -= 1
-                    x0,x1 = _pdtime2index(self.parent(), pd.Series([vt0,vt1]), any_end=True)
-                    self.update_y_zoom(x0, x1)
+                    if self.parent():
+                        x0,x1 = _pdtime2index(self.parent(), pd.Series([vt0,vt1]), any_end=True)
+                        self.update_y_zoom(x0, x1)
             self.updating_linked = False
 
     def zoom_rect(self, vr, scale_fact, center):
@@ -747,7 +776,8 @@ class FinViewBox(pg.ViewBox):
         _mouse_moved(self.win, None)
 
     def update_y_zoom(self, x0=None, x1=None):
-        if self.datasrc is None:
+        datasrc = self.datasrc_or_standalone
+        if datasrc is None:
             return
         if x0 is None or x1 is None:
             tr = self.targetRect()
@@ -755,7 +785,7 @@ class FinViewBox(pg.ViewBox):
             x1 = tr.right()
         # make edges rigid
         xl = max(round(x0-side_margin)+side_margin, -side_margin)
-        xr = min(round(x1-side_margin)+side_margin, self.datasrc.xlen-side_margin)
+        xr = min(round(x1-side_margin)+side_margin, datasrc.xlen-side_margin)
         dxl = xl-x0
         dxr = xr-x1
         if dxl > 0:
@@ -763,9 +793,9 @@ class FinViewBox(pg.ViewBox):
         if dxr < 0:
             x0 += dxr
         x0 = max(round(x0-side_margin)+side_margin, -side_margin)
-        x1 = min(round(x1-side_margin)+side_margin, self.datasrc.xlen-side_margin)
+        x1 = min(round(x1-side_margin)+side_margin, datasrc.xlen-side_margin)
         # fetch hi-lo and set range
-        _,_,hi,lo,cnt = self.datasrc.hilo(x0, x1)
+        _,_,hi,lo,cnt = datasrc.hilo(x0, x1)
         vr = self.viewRect()
         if (x1-x0) < vr.width() and cnt < max_zoom_points:
             return
@@ -785,6 +815,8 @@ class FinViewBox(pg.ViewBox):
             base = (hi+lo) * self.v_zoom_baseline
             y0 = base - rng*self.v_zoom_baseline
             y1 = base + rng*(1-self.v_zoom_baseline)
+        if not self.x_indexed:
+            x0,x1 = _xminmax(datasrc, x_indexed=False, extra_margin=0)
         self.set_range(x0, y0, x1, y1)
 
     def set_range(self, x0, y0, x1, y1):
@@ -1106,8 +1138,7 @@ def create_plot(title='Finance Plot', rows=1, init_zoom_periods=1e10, maximize=T
     win = FinWindow(title)
     # normally first graph is of higher significance, so enlarge
     win.ci.layout.setRowStretchFactor(0, top_graph_scale)
-    if maximize:
-        win.showMaximized()
+    win.show_maximized = maximize
     ax0 = axs = create_plot_widget(master=win, rows=rows, init_zoom_periods=init_zoom_periods, yscale=yscale)
     axs = axs if type(axs) in (tuple,list) else [axs]
     for ax in axs:
@@ -1134,7 +1165,7 @@ def create_plot_widget(master, rows=1, init_zoom_periods=1e10, yscale='linear'):
         else:
             viewbox.setFocus()
         axs += [ax]
-    if isinstance(master, FinWindow):
+    if isinstance(master, pg.GraphicsLayoutWidget):
         proxy = pg.SignalProxy(master.scene().sigMouseMoved, rateLimit=144, slot=partial(_mouse_moved, master))
     else:
         proxy = []
@@ -1342,7 +1373,7 @@ def plot(x, y=None, color=None, width=1, ax=None, style=None, legend=None, zooms
     _set_datasrc(ax, datasrc)
     if legend is not None:
         _create_legend(ax)
-    x = datasrc.x if datasrc.standalone or not ax.x_indexed else datasrc.index
+    x = datasrc.x if datasrc.standalone or not ax.vb.x_indexed else datasrc.index
     y = datasrc.y / ax.vb.yscale.scalef
     if style is None or any(ch in style for ch in '-_.'):
         connect_dots = 'finite' # same as matplotlib; use datasrc.standalone=True if you want to keep separate intervals on a plot
@@ -1500,7 +1531,7 @@ def timer_callback(update_func, seconds, single_shot=False):
     timer.timeout.connect(update_func)
     if single_shot:
         timer.setSingleShot(True)
-    timer.start(seconds*1000)
+    timer.start(int(seconds*1000))
     timers.append(timer)
 
 
@@ -1520,12 +1551,16 @@ def show(qt_exec=True):
             if _loadwindata(win):
                 continue
         for vb in vbs:
-            if vb.datasrc and (vb.linkedView(0) is None or vb.linkedView(0).datasrc is None):
-                vb.update_y_zoom(vb.datasrc.init_x0, vb.datasrc.init_x1)
+            datasrc = vb.datasrc_or_standalone
+            if datasrc and (vb.linkedView(0) is None or vb.linkedView(0).datasrc is None):
+                vb.update_y_zoom(datasrc.init_x0, datasrc.init_x1)
     _repaint_candles()
     for win in windows:
         if isinstance(win, FinWindow) or qt_exec:
-            win.show()
+            if win.show_maximized:
+                win.showMaximized()
+            else:
+                win.show()
     if windows and qt_exec:
         global last_ax, app
         app = QtGui.QApplication.instance()
@@ -1622,7 +1657,7 @@ def _create_plot(ax=None, **kwargs):
 
 
 def _add_timestamp_plot(master, prev_ax, viewbox, index, yscale):
-    native_win = isinstance(master, FinWindow)
+    native_win = isinstance(master, pg.GraphicsLayoutWidget)
     if native_win and prev_ax is not None:
         prev_ax.set_visible(xaxis=False) # hide the whole previous axis
     axes = {'bottom': EpochAxisItem(vb=viewbox, orientation='bottom'),
@@ -1647,7 +1682,6 @@ def _add_timestamp_plot(master, prev_ax, viewbox, index, yscale):
     ax.decouple = partial(_ax_decouple, ax)
     ax.disable_x_index = partial(_ax_disable_x_index, ax)
     ax.prev_ax = prev_ax
-    ax.x_indexed = True
     if index%2:
         viewbox.setBackgroundColor(odd_plot_background)
     viewbox.setParent(ax)
@@ -1655,7 +1689,6 @@ def _add_timestamp_plot(master, prev_ax, viewbox, index, yscale):
 
 
 def _overlay(ax, scale=0.25):
-    global overlay_axs
     viewbox = FinViewBox(ax.vb.win, init_steps=ax.vb.init_steps, yscale=YScale('linear', 1))
     viewbox.v_zoom_scale = scale
     if hasattr(ax, 'ax_widget'):
@@ -1668,7 +1701,6 @@ def _overlay(ax, scale=0.25):
     axo = pg.PlotItem()
     axo.significant_decimals = significant_decimals
     axo.significant_eps = significant_eps
-    axo.x_indexed = ax.x_indexed
     axo.vb = viewbox
     axo.prev_ax = None
     axo.decouple = partial(_ax_decouple, axo)
@@ -1677,20 +1709,21 @@ def _overlay(ax, scale=0.25):
     axo.hideAxis('bottom')
     axo.hideButtons()
     viewbox.addItem(axo)
-    viewbox.setParent(axo)
     ax.vb.sigResized.connect(updateView)
     overlay_axs.append(axo)
     updateView()
     return axo
 
 
-def _ax_set_visible(ax, crosshair=None, xaxis=None, yaxis=None):
+def _ax_set_visible(ax, crosshair=None, xaxis=None, yaxis=None, yaxis_labels=None):
     if crosshair == False:
         ax.crosshair.hide()
-    if yaxis is not None:
-        ax.axes['left']['item'].hide_strings = not yaxis
     if xaxis is not None:
         (ax.showAxis if xaxis else ax.hideAxis)('bottom')
+    if yaxis is not None:
+        (ax.showAxis if yaxis else ax.hideAxis)('left')
+    if yaxis_labels is not None:
+        ax.axes['left']['item'].hide_strings = not yaxis_labels
 
 
 def _ax_decouple(ax):
@@ -1700,7 +1733,7 @@ def _ax_decouple(ax):
 
 
 def _ax_disable_x_index(ax, decouple=True):
-    ax.x_indexed = False
+    ax.vb.x_indexed = False
     if decouple:
         _ax_decouple(ax)
 
@@ -1718,9 +1751,9 @@ def _update_significants(ax, datasrc, force):
     if force or (default_dec and default_eps):
         try:
             sd,se = datasrc.calc_significant_decimals()
-            if default_dec or sd > ax.significant_decimals:
+            if force or default_dec or sd > ax.significant_decimals:
                 ax.significant_decimals = sd
-            if default_eps or se < ax.significant_eps:
+            if force or default_eps or se < ax.significant_eps:
                 ax.significant_eps = se
         except:
             pass # datasrc probably full av NaNs
@@ -1779,9 +1812,10 @@ def _set_datasrc(ax, datasrc):
             _set_x_limits(ax, datasrc)
             viewbox.set_datasrc(viewbox.datasrc) # update zoom
     else:
+        viewbox.standalones.append(datasrc)
         datasrc.update_init_x(viewbox.init_steps)
-        if not ax.x_indexed:
-            _set_x_limits(ax, datasrc)
+        ## if not viewbox.x_indexed:
+            ## _set_x_limits(ax, datasrc)
     # update period if this datasrc has higher resolution
     global epoch_period
     if epoch_period > 1e7 or not datasrc.standalone:
@@ -1901,6 +1935,7 @@ def _update_data(preadjustfunc, adjustfunc, item, ds):
             for c in c1:
                 df_clipped[c] = i.datasrc.df[c]
         i.datasrc.set_df(df_clipped)
+    update_sigdig = False
     if not item.datasrc.standalone:
         # new limits when extending/reducing amount of data
         x_min,x1 = _set_x_limits(item.ax, item.datasrc)
@@ -1909,7 +1944,11 @@ def _update_data(preadjustfunc, adjustfunc, item, ds):
         x0 = x1 - tr.width()
         if tr.right() < x1 - 5 - 2*right_margin_candles:
             x0 = x1 = None
+        prev_top = item.ax.vb.targetRect().top()
         item.ax.vb.update_y_zoom(x0, x1)
+        this_top = item.ax.vb.targetRect().top()
+        if this_top and not (0.99 < abs(prev_top/this_top) < 1.01):
+            update_sigdig = True
         if item.ax.axes['bottom']['item'].isVisible(): # update axes if visible
             item.ax.axes['bottom']['item'].hide()
             item.ax.axes['bottom']['item'].show()
@@ -1918,6 +1957,8 @@ def _update_data(preadjustfunc, adjustfunc, item, ds):
     if isinstance(item, FinPlotItem):
         item.ax.addItem(item)
         item.repaint()
+    if update_sigdig:
+        _update_significants(item.ax, item.ax.vb.datasrc, force=True)
 
 
 def _pre_process_data(vb):
@@ -1930,18 +1971,29 @@ def _pre_process_data(vb):
 
 
 def _set_x_limits(ax, datasrc):
-    if ax.x_indexed:
-        x0 = -side_margin
-        x1 = datasrc.xlen - 1 + side_margin + right_margin_candles # add another margin to get the "snap back" sensation
+    x0,x1 = _xminmax(datasrc, x_indexed=ax.vb.x_indexed)
+    ax.setLimits(xMin=x0, xMax=x1)
+    return x0,x1
+
+
+def _xminmax(datasrc, x_indexed, init_steps=None, extra_margin=0):
+    if x_indexed and init_steps:
+        # initial zoom
+        x0 = max(datasrc.xlen-init_steps, 0) - side_margin - extra_margin
+        x1 = datasrc.xlen - side_margin + extra_margin
+    elif x_indexed:
+        # total x size for indexed data
+        x0 = -side_margin - extra_margin
+        x1 = datasrc.xlen - 1 + side_margin + right_margin_candles + extra_margin# add another margin to get the "snap back" sensation
     else:
+        # x size for plain Y-over-X data (i.e. not indexed)
         x0 = datasrc.x.min()
         x1 = datasrc.x.max()
         # extend margin on decoupled plots
-        d = (x1-x0) * 0.2
+        d = (x1-x0) * (0.2+extra_margin)
         x0 -= d
         x1 += d
-    ax.setLimits(xMin=x0, xMax=x1)
-    return x0, x1
+    return x0,x1
 
 
 def _repaint_candles():
@@ -1999,6 +2051,8 @@ def _mouse_clicked(vb, ev):
 
 
 def _mouse_moved(master, evs):
+    if hasattr(master, 'closing') and master.closing:
+        return
     if not evs:
         evs = master_data[master]['last_mouse_evs']
         if not evs:
@@ -2013,8 +2067,8 @@ def _mouse_moved(master, evs):
     master_data[master]['last_mouse_y'] = y
     # apply to all crosshairs
     for ax in master.axs:
-        point = ax.vb.mapSceneToView(pos)
-        if ax.crosshair:
+        if ax.isVisible() and ax.crosshair:
+            point = ax.vb.mapSceneToView(pos)
             ax.crosshair.update(point)
 
 
@@ -2038,7 +2092,8 @@ def _inspect_pos(ax, inspector, poss):
     try:
         t = ax.vb.datasrc.closest_time(t)
     except KeyError: # when clicking beyond right_margin_candles
-        return
+        if clamp_grid:
+            t = ax.vb.datasrc.x.iloc[-1 if t > 0 else 0]
     try:
         inspector(t, point.y())
     except Exception as e:
@@ -2048,7 +2103,7 @@ def _inspect_pos(ax, inspector, poss):
 def brighten(color, f):
     if not color:
         return color
-    return pg.mkColor(color).lighter(f*100)
+    return pg.mkColor(color).lighter(int(f*100))
 
 
 def _get_color(ax, style, wanted_color):
@@ -2091,10 +2146,17 @@ def _pdtime2index(ax, ts, any_end=False, require_time=False):
             ts = ts.astype('float64') / 1e3
         elif h < 1e10: # handle s epochs
             ts = ts.astype('float64') * 1e3
-    r = []
+    
     datasrc = _get_datasrc(ax)
+    xs = datasrc.x
+
+    # try exact match before approximate match
+    exact = datasrc.index[xs.isin(ts)].to_list()
+    if len(exact) == len(ts):
+        return exact
+    
+    r = []
     for i,t in enumerate(ts):
-        xs = datasrc.x
         xss = xs.loc[xs>t]
         if len(xss) == 0:
             t0 = xs.iloc[-1]
@@ -2115,7 +2177,7 @@ def _pdtime2index(ax, ts, any_end=False, require_time=False):
 
 
 def _get_datasrc(ax, require=True):
-    if ax.vb.datasrc is not None or not ax.x_indexed:
+    if ax.vb.datasrc is not None or not ax.vb.x_indexed:
         return ax.vb.datasrc
     vbs = [ax.vb for win in windows for ax in win.axs]
     for vb in vbs:
@@ -2126,11 +2188,11 @@ def _get_datasrc(ax, require=True):
 
 
 def _x2local_t(datasrc, x):
-    return _x2t(datasrc, x, lambda t: datetime.fromtimestamp(t/1000).isoformat().replace('T',' '))
+    return _x2t(datasrc, x, lambda t: datetime.fromtimestamp(t/1000, tz=display_timezone).isoformat(sep=' ').split('+')[0])
 
 
 def _x2utc(datasrc, x):
-    return _x2t(datasrc, x, lambda t: datetime.utcfromtimestamp(t/1000).isoformat().replace('T',' '))
+    return _x2t(datasrc, x, lambda t: datetime.utcfromtimestamp(t/1000).isoformat(sep=' '))
 
 
 def _x2t(datasrc, x, ts2str):
@@ -2171,7 +2233,7 @@ def _round_to_significant(rng, rngmax, x, significant_decimals, significant_eps)
     if is_highres and abs(x)>0:
         exp10 = floor(np.log10(abs(x)))
         x = x / (10**exp10)
-        sd = min(5, sd+int(np.log10(rngmax)))
+        sd = min(3, sd+int(np.log10(rngmax)))
         fmt = '%%%i.%ife%%i' % (sd, sd)
         r = fmt % (x, exp10)
     else:
@@ -2194,7 +2256,10 @@ def _roihandle_move_snap(vb, orig_func, pos, modifiers=QtCore.Qt.KeyboardModifie
 
 def _clamp_xy(ax, x, y):
     y = ax.vb.yscale.xform(y)
-    if clamp_grid and ax.x_indexed:
+    if clamp_grid and ax.vb.x_indexed:
+        ds = ax.vb.datasrc
+        if x < 0 or (ds and x > len(ds.df)-1):
+            x = 0 if x < 0 else len(ds.df)-1
         x = round(x)
         eps = ax.significant_eps
         eps2 = np.sign(y) * 0.5 * eps
